

__author__ = 'scribblemaniac'

from renderchan.module import RenderChanModule
from renderchan.utils import which
import subprocess
import os
import re
import random

class RenderChanGimpModule(RenderChanModule):
    def __init__(self):
        RenderChanModule.__init__(self)
        self.conf['binary']=self.findBinary("gimp")
        self.conf["packetSize"]=0
        self.extraParams['use_own_dimensions'] = '1'

    def getInputFormats(self):
        return ["xcf", "fli", "flc", "dds", "dcm", "dicom", "eps", "fit", "fits", "g3", "xjt", "cel", "wmf", "ico", "pnm", "ppm", "pgm", "pbm", "psp", "psd", "pdf", "ps", "tiff", "bmp", "xbm", "xwd", "pcx", "pcc", "ora"]

    def getOutputFormats(self):
        return ["png", "jpg", "jpeg", "pdf", "psd", "tif", "tiff", "bmp", "ico", "txt", "html", "gif", "mng"]

    def render(self, filename, outputPath, startFrame, endFrame, format, updateCompletion, extraParams={}):

        comp=0.0
        updateCompletion(comp)

        # Determines drawable (depending on layer support)
        if format in ("gif", "ico", "mng", "psd"):
            drawable="gimp-image-get-active-drawable image"
        else:
            drawable="gimp-image-merge-visible-layers image CLIP-TO-IMAGE"

        # Preprocessing procedures
        if format == "gif":
            preprocedure="(gimp-image-convert-indexed image 0 0 255 0 0 \"\")"
        else:
            preprocedure=""

        # Determine procedure name
        if format in ("png", "tiff"):
            saveProcedure="file-%s-save2" % format
        elif format in ("txt", "html"):
            saveProcedure="file-aa-save"
        elif format == "jpg":
            saveProcedure="file-jpeg-save"
        elif format == "tif":
            saveProcedure="file-tiff-save2"
        elif format in ("jpeg", "pdf", "psd", "gif", "mng"):
            saveProcedure="file-%s-save" % format
        else:
            saveProcedure="gimp-file-save"

        # Determine paramters
        if format == "png":
            # PNG get special treatment because default parameters can be fetched from GIMP
            #saveParameters=" ".join(subprocess.check_output([self.conf['binary'], "-i", "-b", "(let ((str \"\") (defaults (file-png-get-defaults))) (gimp-message (do ((i 0 (+ i 1))) ((= i 9) str) (set! str (string-append str (number->string (nth i defaults)))))))", "-b", "(gimp-quit 0)"], stderr=subprocess.STDOUT)[19:28])
            #print saveParameters
            saveParameters="0 9 0 0 0 1 0 0 0"
        elif format in ("jpg", "jpeg"):
            saveParameters=".9 0 0 0 \" \" 0 1 0 1"
        elif format == "pdf":
            saveParameters="0 1 1"
        elif format == "psd":
            saveParameters="1 1"
        elif format in ("tif", "tiff"):
            saveParameters="1 0"
        elif format == "txt":
            saveParameters="\"Text file\""
        elif format == "html":
            saveParameters="\"Pure html\""
        elif format == "gif":
            saveParameters="0 1 100 2"
        elif format == "mng":
            saveParameters="0 9 .9 0 1 100 2 1 0 0 1 0"
        else:
            saveParameters=""

        if extraParams['use_own_dimensions']:
            width_arg = '(car (gimp-image-width image))'
            height_arg = '(car (gimp-image-height image))'
        else:
            width_arg = extraParams['width']
            width_arg = extraParams['height']
        # See docs for readable script-fu code
        commandline=[self.conf['binary'], "-i", "-b", "(let*  ((filename \"%s\") (outpath \"%s\") (image (car (gimp-file-load RUN-NONINTERACTIVE filename filename))) (drawable (car (%s)))) %s (gimp-image-scale image %s %s) (%s RUN-NONINTERACTIVE image drawable outpath outpath %s) (gimp-image-delete image))" % (filename, outputPath, drawable, preprocedure, width_arg, height_arg, saveProcedure, saveParameters), "-b", "(gimp-quit 0)"]

        subprocess.check_call(commandline)

        updateCompletion(1.0)
